@@ -156,6 +156,16 @@ class TestBarbicanManage(TestBarbicanManageBase):
             ['barbican.cmd.barbican_manage', 'db', 'current'], mock_current,
             False, sql_url='mockdburl')
 
+    @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
+    def test_hsm_check_mkek(self, mock_pkcs11):
+        mock_pkcs11.return_value.get_session.return_value = 1
+        mock_pkcs11.return_value.get_key_handle.return_value = 1
+        mock_getkey = mock_pkcs11.return_value.get_key_handle
+        self._main_test_helper(
+            ['barbican.cmd.barbican_manage', 'hsm', 'check_mkek',
+             '--library-path', 'mocklib', '--passphrase', 'mockpassewd',
+             '--label', 'mocklabel'], mock_getkey, 'CKK_AES', 'mocklabel', 1)
+
     @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
     def test_hsm_gen_mkek(self, mock_pkcs11):
         mock_pkcs11.return_value.get_session.return_value = 1
@@ -165,8 +175,9 @@ class TestBarbicanManage(TestBarbicanManageBase):
         self._main_test_helper(
             ['barbican.cmd.barbican_manage', 'hsm', 'gen_mkek',
              '--library-path', 'mocklib', '--passphrase', 'mockpassewd',
-             '--label', 'mocklabel'], mock_genkey,
-            32, 1, 'mocklabel', encrypt=True, wrap=True, master_key=True)
+             '--label', 'mocklabel'], mock_genkey, 'CKK_AES',
+            32, 'CKM_AES_KEY_GEN', 1, 'mocklabel', encrypt=True, wrap=True,
+            master_key=True)
 
     @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
     def test_hsm_gen_hmac(self, mock_pkcs11):
@@ -177,8 +188,8 @@ class TestBarbicanManage(TestBarbicanManageBase):
         self._main_test_helper(
             ['barbican.cmd.barbican_manage', 'hsm', 'gen_hmac',
              '--library-path', 'mocklib', '--passphrase', 'mockpassewd',
-             '--label', 'mocklabel'], mock_genkey,
-            32, 1, 'mocklabel', sign=True, master_key=True)
+             '--label', 'mocklabel'], mock_genkey, 'CKK_AES',
+            32, 'CKM_AES_KEY_GEN', 1, 'mocklabel', sign=True, master_key=True)
 
     @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
     def test_hsm_gen_mkek_non_default_length(self, mock_pkcs11):
@@ -190,8 +201,18 @@ class TestBarbicanManage(TestBarbicanManageBase):
             ['barbican.cmd.barbican_manage', 'hsm', 'gen_mkek',
              '--length', '48', '--library-path', 'mocklib',
              '--passphrase', 'mockpassewd', '--label', 'mocklabel'],
-            mock_genkey, 48, 1, 'mocklabel', encrypt=True, wrap=True,
-            master_key=True)
+            mock_genkey, 'CKK_AES', 48, 'CKM_AES_KEY_GEN', 1, 'mocklabel',
+            encrypt=True, wrap=True, master_key=True)
+
+    @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
+    def test_hsm_check_hmac(self, mock_pkcs11):
+        mock_pkcs11.return_value.get_session.return_value = 1
+        mock_pkcs11.return_value.get_key_handle.return_value = 1
+        mock_getkey = mock_pkcs11.return_value.get_key_handle
+        self._main_test_helper(
+            ['barbican.cmd.barbican_manage', 'hsm', 'check_hmac',
+             '--library-path', 'mocklib', '--passphrase', 'mockpassewd',
+             '--label', 'mocklabel'], mock_getkey, 'CKK_AES', 'mocklabel', 1)
 
     @mock.patch('barbican.plugin.crypto.pkcs11.PKCS11')
     def test_hsm_gen_hmac_non_default_length(self, mock_pkcs11):
@@ -203,4 +224,5 @@ class TestBarbicanManage(TestBarbicanManageBase):
             ['barbican.cmd.barbican_manage', 'hsm', 'gen_hmac',
              '--length', '48', '--library-path', 'mocklib',
              '--passphrase', 'mockpassewd', '--label', 'mocklabel'],
-            mock_genkey, 48, 1, 'mocklabel', sign=True, master_key=True)
+            mock_genkey, 'CKK_AES', 48, 'CKM_AES_KEY_GEN', 1, 'mocklabel',
+            sign=True, master_key=True)
